/*
 * Copyright 2014 Dmytro Titov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.github.autsia.crowly.services.scheduling.impl;

import io.github.autsia.crowly.model.Campaign;
import io.github.autsia.crowly.repositories.CampaignRepository;
import io.github.autsia.crowly.repositories.MentionRepository;
import io.github.autsia.crowly.services.scheduling.CampaignsManager;
import io.github.autsia.crowly.services.scheduling.jobs.impl.CrawlingJobImpl;
import io.github.autsia.crowly.services.social.SocialEndpointsFacade;
import org.apache.log4j.Logger;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.List;

import static org.quartz.CronScheduleBuilder.cronSchedule;
import static org.quartz.JobBuilder.newJob;
import static org.quartz.TriggerBuilder.newTrigger;

/**
 * Created by Dmytro on 4/25/2014 at 22:11
 * Project: crowly
 */
@Service
public class CampaignsManagerImpl implements CampaignsManager {

    public static final String CAMPAIGN_KEY = "campaign";
    public static final String SOCIAL_ENDPOINTS_FACADE_KEY = "socialEndpointsFacade";
    public static final String MENTION_REPOSITORY_KEY = "mentionRepository";
    private static final Logger logger = Logger.getLogger(CampaignsManagerImpl.class);
    protected CampaignRepository campaignRepository;
    protected Scheduler scheduler;
    protected SocialEndpointsFacade socialEndpointsFacade;
    protected MentionRepository mentionRepository;

    @Override
    @PostConstruct
    public void init() {
        try {
            startAll(campaignRepository.findByStatus(Campaign.ACTIVE));
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    @Override
    public void startCampaign(Campaign campaign) {
        JobDetail jobDetail = createJobDetail(campaign);
        Trigger trigger = createTrigger(campaign, jobDetail);
        try {
            scheduler.scheduleJob(jobDetail, trigger);
        } catch (SchedulerException e) {
            logger.error(e.getMessage(), e);
        }
    }

    @Override
    public void stopCampaign(Campaign campaign) {
        try {
            scheduler.unscheduleJob(new TriggerKey(campaign.getId(), campaign.getName()));
        } catch (SchedulerException e) {
            logger.error(e.getMessage(), e);
        }
    }

    @Override
    public void startAll(List<Campaign> campaigns) {
        for (Campaign campaign : campaigns) {
            startCampaign(campaign);
        }
    }

    @Override
    public void stopAll(List<Campaign> campaigns) {
        for (Campaign campaign : campaigns) {
            stopCampaign(campaign);
        }
    }

    protected JobDetail createJobDetail(Campaign campaign) {
        JobDetail jobDetail = newJob(CrawlingJobImpl.class)
                .withIdentity(campaign.getId(), campaign.getName())
                .build();

        jobDetail.getJobDataMap().put(CAMPAIGN_KEY, campaign);
        jobDetail.getJobDataMap().put(SOCIAL_ENDPOINTS_FACADE_KEY, socialEndpointsFacade);
        jobDetail.getJobDataMap().put(MENTION_REPOSITORY_KEY, mentionRepository);

        return jobDetail;
    }

    protected Trigger createTrigger(Campaign campaign, JobDetail jobDetail) {
        return newTrigger()
                .withIdentity(campaign.getId(), campaign.getName())
                .withSchedule(cronSchedule(campaign.getCronExpression()))
                .forJob(jobDetail)
                .build();
    }

    @Autowired
    public void setCampaignRepository(CampaignRepository campaignRepository) {
        this.campaignRepository = campaignRepository;
    }

    @Autowired
    public void setScheduler(Scheduler scheduler) {
        this.scheduler = scheduler;
    }

    @Autowired
    public void setSocialEndpointsFacade(SocialEndpointsFacade socialEndpointsFacade) {
        this.socialEndpointsFacade = socialEndpointsFacade;
    }

    @Autowired
    public void setMentionRepository(MentionRepository mentionRepository) {
        this.mentionRepository = mentionRepository;
    }
}
